package handler

import (
	"context"
	"net/http"

	"git.unistack.org/unistack-org/pkgdash/internal/models"
	pb "git.unistack.org/unistack-org/pkgdash/proto"
	httpsrv "go.unistack.org/micro-server-http/v4"
	"go.unistack.org/micro/v4/logger"
)

func (h *Handler) CommentList(ctx context.Context, req *pb.CommentListReq, rsp *pb.CommentListRsp) error {
	logger.Debug(ctx, "Start GetModule")

	err := req.Validate()
	if err != nil {
		logger.Error(ctx, "validate error", err)
		httpsrv.SetRspCode(ctx, http.StatusBadRequest)
		return httpsrv.SetError(NewValidationError(err))
	}

	comments, err := h.store.CommentList(ctx, req)
	if err != nil {
		logger.Error(ctx, "comment list error", err)
		httpsrv.SetRspCode(ctx, http.StatusInternalServerError)
		return httpsrv.SetError(NewInternalError(err))
	}

	for _, com := range comments {
		rsp.Comments = append(rsp.Comments, models.NewComment(com))
	}

	logger.Debug(ctx, "Success finish getModule")
	return nil
}
