package handler

import (
	"context"
	"net/http"

	"git.unistack.org/unistack-org/pkgdash/internal/models"
	pb "git.unistack.org/unistack-org/pkgdash/proto"
	httpsrv "go.unistack.org/micro-server-http/v4"
	"go.unistack.org/micro/v4/logger"
)

func (h *Handler) PackageCreate(ctx context.Context, req *pb.PackageCreateReq, rsp *pb.PackageCreateRsp) error {
	logger.Debug(ctx, "PackagesCreate handler start")

	if err := req.Validate(); err != nil {
		logger.Error(ctx, "validate error", err)
		httpsrv.SetRspCode(ctx, http.StatusBadRequest)
		return httpsrv.SetError(NewValidationError(err))
	}

	pkg, err := h.store.PackageCreate(ctx, req)
	if err != nil {
		logger.Error(ctx, "package create error", err)
		httpsrv.SetRspCode(ctx, http.StatusBadRequest)
		return httpsrv.SetError(NewValidationError(err))
	}

	rsp.Package = models.NewPackage(pkg)

	logger.Debug(ctx, "PackagesCreate handler stop")
	return nil
}
